#include "AMDiS.h"
#include "MpCCIAdapter.h"
#include "MultiGridSolver.h"
#include "GSSmoother.h"

using namespace std;
using namespace AMDiS;

// ===========================================================================
// ===== function definitions ================================================
// ===========================================================================

/// Dirichlet boundary function
class G : public AbstractFunction<double, WorldVector<double> >
{
public:
  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const
  {
    return exp(-10.0 * (x * x));
  }
};

/// RHS function
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {}

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    int dim = x.getSize();
    double r2 = x * x;
    double ux = exp(-10.0 * r2);
    return -(400.0 * r2 - 20.0 * dim) * ux;
  }
};

class WaitingProblem : public ProblemScal
{
public:
  WaitingProblem(const char *name) 
    : ProblemScal(name) 
  {}

  void solve(AdaptInfo *adaptInfo) {
    FUNCNAME("WaitingProblem::solve()");

    static MultiGridSolver *mgSolver = NULL;

    if (!mgSolver) 
      mgSolver = new MultiGridSolver(feSpace_, 
				     new GSSmoother(1.0),
				     systemMatrix_,
				     solution_,
				     rhs_);

    mgSolver->solve();
  }
};

// ===========================================================================
// ===== main program ========================================================
// ===========================================================================

int main(int argc, char* argv[])
{
  FUNCNAME("main");

  // ===== check for init file =====
  TEST_EXIT(argc == 2)("usage: mpcci1 initfile\n");

  // ===== init parameters =====
  Parameters::init(false, argv[1]);

  // ===== create and init the scalar problem ===== 
  WaitingProblem mpcci1("mpcci1");

  mpcci1.initialize(INIT_ALL);

  // === create adapt info ===
  AdaptInfo *adaptInfo = new AdaptInfo("mpcci1->adapt", 1);

  // === create adapt ===
  AdaptStationary *adapt = new AdaptStationary("mpcci1->adapt",
					       &mpcci1,
					       adaptInfo);
  
  //mpcci1.setAdapt(adapt);

  // ===== add boundary conditions =====
  mpcci1.addDirichletBC(1, new G);

  // ===== create matrix operator =====
  Operator matrixOperator(Operator::MATRIX_OPERATOR, mpcci1.getFESpace());
  matrixOperator.addSecondOrderTerm(new Laplace_SOT);
  mpcci1.addMatrixOperator(&matrixOperator);

  // ===== create rhs operator =====
  int degree = mpcci1.getFESpace()->getBasisFcts()->getDegree();
  Operator rhsOperator(Operator::VECTOR_OPERATOR, mpcci1.getFESpace());
  rhsOperator.addZeroOrderTerm(new CoordsAtQP_ZOT(new F(degree)));
  mpcci1.addVectorOperator(&rhsOperator);

  // ===== start adaption loop =====
  adapt->adapt();
}


