#include "AMDiS.h"
#include "mpi.h"
#include "ParallelProblem.h"

using namespace std;
using namespace AMDiS;

/// Dirichlet boundary function
class G : public AbstractFunction<double, WorldVector<double> >
{
public:

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    return exp(-10.0*(x*x));
  }
};

class GrdG : public AbstractFunction<WorldVector<double>, WorldVector<double> >
{
  /// Implementation of AbstractFunction::operator().
  WorldVector<double> operator()(const WorldVector<double>& x) const 
  {
    return x * -20.0 * exp(-10.0*(x*x));
  }
};

/// RHS function
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {}

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    int dim = x.getSize();
    double r2 = (x*x);
    double ux = exp(-10.0*r2);
    return -(400.0*r2 - 20.0*dim)*ux;
  }
};

// ===== main program ===== 
int main(int argc, char* argv[])
{
  MPI::Init(argc, argv);

  FUNCNAME("main");

  // ===== check for init file =====
  TEST_EXIT(argc >= 2)("usage: ellipt initfile\n");

  // ===== init parameters =====
  Parameters::init(false, argv[1]);

  // ===== create and init the scalar problem ===== 
  ProblemScal ellipt("ellipt");
  ellipt.initialize(INIT_ALL);

  // === parallel problem ===
  std::vector<DOFVector<double>*> vectors;
  ParallelProblemScal parallelellipt("ellipt->parallel", &ellipt, NULL, vectors);

  // === create adapt info ===
  AdaptInfo *adaptInfo = new AdaptInfo("ellipt->adapt", 1);

  // === create adapt ===
  AdaptStationary *adapt = new AdaptStationary("ellipt->adapt",
					       &parallelellipt,
					       adaptInfo);
  
  // ===== add boundary conditions =====
  ellipt.addDirichletBC(1, new G);

  // ===== create matrix operator =====
  Operator matrixOperator(Operator::MATRIX_OPERATOR, ellipt.getFESpace());
  matrixOperator.addSecondOrderTerm(new Laplace_SOT);
  ellipt.addMatrixOperator(&matrixOperator);

  // ===== create rhs operator =====
  int degree = ellipt.getFESpace()->getBasisFcts()->getDegree();
  Operator rhsOperator(Operator::VECTOR_OPERATOR, ellipt.getFESpace());
  rhsOperator.addZeroOrderTerm(new CoordsAtQP_ZOT(new F(degree)));
  ellipt.addVectorOperator(&rhsOperator);

  parallelellipt.initParallelization(adaptInfo);
  adapt->adapt();
  parallelellipt.exitParallelization(adaptInfo);

  MPI::Finalize();
}


